/*
 * Copyright (c) 2010-2012 by inovat, innovative systeme - verkehr - tunnel - technik, Dipl.-Ing. H. C. Kniss
 *
 * This file is part of de.inovat.vew.autoexport.AuftragsListe
 *
 * de.inovat.vew.autoexport.AuftragsListe is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * de.inovat.vew.autoexport.AuftragsListe is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * <p>
 * You should have received a copy of the GNU General Public License
 * along with de.inovat.vew.autoexport.AuftragsListe; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Contact Information:
 * inovat, Dipl.-Ing. H. C. Kniss
 * Koelner Strasse 30
 * D-50859 Koeln
 * +49 (0)2234 4301 800
 * info@invat.de
 * www.inovat.de
 */



package de.inovat.vew.autoexport;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Singletonklasse, welche die Auftragsliste fr Aggegationsauftrge verwaltet.
 * <p/>
 * Die Auftragsliste ermglicht das Hinzufgen von Auftrgen ({@link de.inovat.dua.aggrlzzsdtv.Auftrag} sowie die
 * Entnahme des aktuell auszufhrenden Auftrags. Die Zugriffe auf die Auftragsliste ist intern syncronisiert.
 *
 * @author inovat, innovative systeme - verkehr - tunnel - technik
 * @author Dipl.-Ing. Hans Christian Kni (HCK)
 * @version $Revision: 780 $ / $Date: 2012-10-31 11:44:41 +0100 (Mi, 31 Okt 2012) $ / ($Author: HCK $)
 */
public class AuftragsListe {
    private static AuftragsListe       dieEinzigeInstanz = new AuftragsListe();
    private static final List<Auftrag> _listeAuftraege   = new ArrayList<Auftrag>();

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /** Privater Konstruktor. */
    private AuftragsListe() {}

    //~ GET METHODEN ==========================================================

    /**
     * Liefert die Instanz des Singletons.
     *
     * @return Instanz des Singletons
     */
    public static AuftragsListe getInstanz() {
        return dieEinzigeInstanz;
    }

    //~ METHODEN ==============================================================

    /**
     * Fgt einen neuen Auftrag zur Liste der Auftrge hinzu. Die Liste wird intern so sortiert, dass die Methode {@link
     * #holeNaechstenAuftrag()} immer den zum aktuellen Zeitpunkt nchsten auszufhrenden Auftrag zurckliefert.
     *
     * @param auftrag Neuer Auftrag, der in die Liste der Auftrge eingetragen werden soll.
     */
    public void addAuftrag(Auftrag auftrag) {
        synchronized (_listeAuftraege) {
            _listeAuftraege.add(auftrag);
            Collections.sort(_listeAuftraege, new Comparator<Auftrag>() {
                @Override
                public int compare(Auftrag auftrag1, Auftrag auftrag2) {

                    // Zeitliche Sortierung der Auftrge nach Ausfhrungszeitpunkt und Auftragsart
                    if (auftrag1.getAusfuehrungsZeitPunkt() < auftrag2.getAusfuehrungsZeitPunkt()) {
                        return -1;
                    }

                    if (auftrag1.getAusfuehrungsZeitPunkt() > auftrag2.getAusfuehrungsZeitPunkt()) {
                        return 1;
                    }

                    return 0;
                }
            });
        }
    }

    /**
     * Gibt zurck, ob noch Auftrge (aktuell oder zuknftig) anstehen.
     *
     * @return <code>true</code>, wenn noch Auftrge (aktuell oder zuknftig) anstehen, andernfalls <code>false</code>.
     */
    public boolean hatNochAuftraege() {
        synchronized (_listeAuftraege) {
            return _listeAuftraege.size() != 0;
        }
    }

    /**
     * Liefert eine Kopie der Auftragsliste.
     *
     * @return Kopie der Auftragsliste.
     */
    public List<Auftrag> holeKopieAuftragsListe() {
        List<Auftrag> listeKopieAuftraege = new ArrayList<Auftrag>();

        synchronized (_listeAuftraege) {
            for (Auftrag auftrag : _listeAuftraege) {
                listeKopieAuftraege.add(auftrag);
            }
        }

        return listeKopieAuftraege;
    }

    /**
     * Liefert den, bezogen auf den aktuellen Zeitpunkt, nchsten auszufhrenden Auftrag zurck.
     *
     * @return Nchster auszufhrender Auftrag oder <code>null</code>, wenn kein Auftrag aktuell auszufhren ist. Der
     *         zurckgegebene Auftrag wird entfernt und kann nicht noch einmal abgerufen werden.
     */
    public Auftrag holeNaechstenAuftrag() {
        Auftrag naechsterAuftrag = null;

        synchronized (_listeAuftraege) {
            if ((_listeAuftraege.size() > 0) && (_listeAuftraege.get(0).getAusfuehrungsZeitPunkt() < System.currentTimeMillis())) {
                naechsterAuftrag = _listeAuftraege.get(0);
                _listeAuftraege.remove(0);
            }
        }

        return naechsterAuftrag;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 31.10.12
